/***************************************************************************/
/*                                                                         */
/*  File: Explode.h                                                        */
/*  Date: 06-02-07                                                         */
/*                                                                         */
/***************************************************************************/
/*
	A simple explosion effect for when a bullet or object hits a bullet,
	such as when we fire and it hits the ground, we want to see something
	happen!..some impact effect....just simple, but can be expanded upon
*/
/***************************************************************************/
#ifndef EXPLODE_H
#define EXPLODE_H


#include <windows.h>    // opengl has issues if we don't include it!
#include <GL/gl.h>
//#include <GL/glu.h>	// gluLookAt(..), glPerspective(..)
#include <stdlib.h>		// rand(..)

#include "Vector3.h"
#include "GLDraw.h"		// DrawUnitCube(..)

extern float g_timeStep;

#define MAX_EXPLODES 5  // Maximum number of explosions at one time
#define MAX_BITS	 10 // Number of bits of pieces for each explosion


#define RandZeroToOne ((float)rand()/32767.0f)
#define RandNegOneToOne (((float)rand()/16383.0f)-1.0f)

#define RandRange(x,y) ( ((RandZeroToOne*(y-x)) + x )) 

/***************************************************************************/
/*                                                                         */
/*  class ExplodeBit                                                       */
/*  What? - We break up the explosion into, different explosions, which    */
/*  can deal with themselves, we take this a bit further, by allowing each */
/*  particle of the explosion to update itself, so if you have 10 bits of  */
/*  shrapnell explode from the ground, then each bit will have its own     */
/*  draw and update function and will look after itself, once its m_life   */
/*  time has ran out, its no longer drawn and dead.                        */
/*                                                                         */
/***************************************************************************/

class ExplodeBit
{
public:
	VECTOR3 m_pos;
	VECTOR3 m_vel;
	float   m_life;

	void Reset(const VECTOR3& pos)
	{
		m_life = 1.0f + RandZeroToOne * 5.0f;
		m_pos  = pos;
		VECTOR3 randvel = VECTOR3(RandNegOneToOne*0.1f,0.0f,RandNegOneToOne*0.1f);
		m_vel  = VECTOR3(0,1,0);

		m_vel += randvel;

		float strength = 5.0f + RandZeroToOne*5.0f;
		m_vel *= strength;
	}

	bool Update()
	{
		if (m_life<0.0f) return false;
		
		m_life -= g_timeStep;
		m_pos += m_vel*g_timeStep;

		m_vel = m_vel*0.998f;

		const VECTOR3 gravity = VECTOR3(0,-9.8f,0);
		m_vel += g_timeStep*gravity;

		return true;
	}

	void Render()
	{
		if (m_life<0.0f) return;

		glPushMatrix();
		glTranslated(m_pos.x,m_pos.y,m_pos.z);
		float sz = 0.1f;
		glScalef (sz, sz, sz);
		glLineWidth(1.0);
		glColor3d(1.0,0.0,0.0);
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		DrawUnitCube();
		glPopMatrix();
		
	}
};// End ExplodeBit class


/***************************************************************************/
/*                                                                         */
/*  class SingleExplode                                                    */
/*  What? - We have multiple explosions going on at the same time, being   */
/*  caused by lots of different tanks or planes or whatever is happening   */
/*  in our world, each explosion has its own class, which is this...so     */
/*  this handles a single explosion and updates and calls render on it     */
/*                                                                         */
/***************************************************************************/

class SingleExplode
{
public:
	bool		m_active;

	ExplodeBit	m_bits[MAX_BITS];

	void Reset(const VECTOR3& pos)
	{
		m_active = true;
		for (int i=0; i<MAX_BITS; i++)
		{
			m_bits[i].Reset(pos);
		}
	}

	void Update()
	{
		m_active = false;
		for (int i=0; i<MAX_BITS; i++)
		{
			m_active |= m_bits[i].Update();
		}
	}

	void Render()
	{
		for (int i=0; i<MAX_BITS; i++)
		{
			m_bits[i].Render();
		}
	}
};// End SingleExplode Class



/***************************************************************************/
/*                                                                         */
/*  class Explode                                                          */
/*  What? - Technically you'd call this an explosion manager, as it keeps  */
/*  track of many explosions, we simply call AddExplode(..) with the       */
/*  position of the explosion, and it takes care of the rest, it create    */
/*  an explosion at that point, and updates it and keeps track of it, till */
/*  it is dead.                                                            */
/*                                                                         */
/***************************************************************************/

class Explode
{
protected:
	SingleExplode m_explodeList[MAX_EXPLODES];

public:
	Explode()
	{
		for (int i=0; i<MAX_EXPLODES; i++)
		{
			m_explodeList[i].m_active=false;
		}
	}

	void AddExplode(const VECTOR3& pos)
	{
		for (int i=0; i<MAX_EXPLODES; i++)
		{
			if (m_explodeList[i].m_active==false)
			{
				m_explodeList[i].Reset(pos);
				m_explodeList[i].m_active=true;
				break;
			}
		}
	}

	void Update()
	{
		for (int i=0; i<MAX_EXPLODES; i++)
		{
			if (m_explodeList[i].m_active)
			{
				m_explodeList[i].Update();
			}
		}
	}

	void Render()
	{
		for (int i=0; i<MAX_EXPLODES; i++)
		{
			if (m_explodeList[i].m_active)
			{
				m_explodeList[i].Render();
			}
		}
	}

};//End Explode Class



#endif //EXPLODE_H