/***************************************************************************/
/*                                                                         */
/*  File: matrix4.h                                                        */
/*                                                                         */
/***************************************************************************/

#ifndef MATRIX4_H
#define MATRIX4_H

#include "Vector3.h" // class VECTOR3


class MATRIX4
{
	public:
    union 
	{
        struct 
		{
            float        _11, _12, _13, _14;
            float        _21, _22, _23, _24;
            float        _31, _32, _33, _34;
            float        _41, _42, _43, _44;

        };
        float m[4][4];
    };


	MATRIX4()
	{
		Identity();
	}

	MATRIX4( float m11, float m12, float m13, float m14,
			 float m21, float m22, float m23, float m24,
			 float m31, float m32, float m33, float m34,
			 float m41, float m42, float m43, float m44 )
	{
		_11=m11; _12=m12; _13=m13; _14=m14;
		_21=m21; _22=m22; _23=m23; _24=m24;
		_31=m31; _32=m32; _33=m33; _34=m34;
		_41=m41; _42=m42; _43=m43; _44=m44;
	}


	MATRIX4 Identity()
	{
		memset(this, 0, sizeof(MATRIX4));
		_11=_22=_33=_44=1.0f;
		return *this;
	}


	void RotateY( const float& a )
	{
		Identity();

		float s = sinf(a);
		float c = cosf(a);

		_11=c;    _12=0.0f;  _13=-s;
		_21=0.0f; _22=1.0f;  _23=0.0f;
		_31=s;    _32=0.0f;  _33=c;
	}

	void RotateX( const float& a )
	{
		Identity();

		float s = sinf(a);
		float c = cosf(a);

		_11=1;    _12=0;     _13=0;
		_21=0;    _22=c;     _23=s;
		_31=0;    _32=-s;    _33=c;

		/*
		_11=1;    _12=0;    _13=0;
		_21=0;    _22=c;    _23=-s;
		_31=0;    _32=s;    _33=c;
		*/
	}

	void RotateZ( const float& a )
	{
		Identity();

		float s = sinf(a);
		float c = cosf(a);

		_11=c;    _12=s;    _13=0;
		_21=-s;   _22=c;    _23=0;
		_31=0;    _32=0;    _33=1;
	}

	static void MatrixMultiply(MATRIX4& out, MATRIX4& inA, MATRIX4& inB)
	{
		MATRIX4 tmp;
		tmp._11 = inA._11*inB._11 + inA._12*inB._21 + inA._13*inB._31;
		tmp._12 = inA._11*inB._12 + inA._12*inB._22 + inA._13*inB._32;
		tmp._13 = inA._11*inB._13 + inA._12*inB._23 + inA._13*inB._33;
		tmp._14 = 0.0f;

		tmp._21 = inA._21*inB._11 + inA._22*inB._21 + inA._23*inB._31;
		tmp._22 = inA._21*inB._12 + inA._22*inB._22 + inA._23*inB._32;
		tmp._23 = inA._21*inB._13 + inA._22*inB._23 + inA._23*inB._33;
		tmp._24 = 0.0f;

		tmp._31 = inA._31*inB._11 + inA._32*inB._21 + inA._33*inB._31;
		tmp._32 = inA._31*inB._12 + inA._32*inB._22 + inA._33*inB._32;
		tmp._33 = inA._31*inB._13 + inA._32*inB._23 + inA._33*inB._33;
		tmp._34 = 0.0f;

		tmp._41 = inA._41*inB._11 + inA._42*inB._21 + inA._43*inB._31;
		tmp._42 = inA._41*inB._12 + inA._42*inB._22 + inA._43*inB._32;
		tmp._43 = inA._41*inB._13 + inA._42*inB._23 + inA._43*inB._33;
		tmp._44 = 1.0f;

		out=tmp;
	}
};



static void TransformCoord(VECTOR3 *vout, VECTOR3*v, MATRIX4 *m )
{
	VECTOR3 tmp;
	tmp.x = VECTOR3::dot(v->x, v->y, v->z,
							m->_11, m->_21, m->_31) + m->_41;

	tmp.y = VECTOR3::dot(v->x, v->y, v->z,
							m->_12, m->_22, m->_32) + m->_42;

	tmp.z = VECTOR3::dot(v->x, v->y, v->z,
							m->_13, m->_23, m->_33) + m->_43;

	//float w = dot(v->x, v->y, v->z,
	//		      m[12], m[13], m[14]) + v[3]*m[15];


	*vout = tmp;
}



#endif //MATRIX4_H